/** @file x11_ledbox.cpp

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	Skelton for retropc emulator

	@author Sasaji
	@date   2016.01.21 -

	@brief [ led box ]
*/

#if !(defined(USE_SDL2) && defined(USE_SDL2_LEDBOX))

#include "x11_ledbox.h"

#if defined(SDL_VIDEO_DRIVER_X11) && defined(USE_X11_LEDBOX)

#include <SDL_syswm.h>

//
// for X window system
//

LedBox::LedBox() : LedBoxBase()
{
	display = NULL;
	screen = 0;
	parent = 0;
	window = 0;
	gc = 0;
	event_masks = 0;
	memset(&image, 0, sizeof(image));

	event_working = false;

	pStart.x = 0;
	pStart.y = 0;
}

LedBox::~LedBox()
{
	destroy_dialog();
}

void LedBox::show_dialog()
{
	if (window) {
		XWindowAttributes attr;
		XEvent event;

		XGetWindowAttributes(display, window, &attr);
		if (visible && !inside) {
			if (attr.map_state == IsUnmapped) {
				XMapRaised(display, window);
				// wait until raised
//				XIfEvent(display, &event, &is_map_notify, (XPointer)&window);
				XSync(display, False);
//				XFlush(display);
			}
		} else {
			if (attr.map_state != IsUnmapped) {
				XWithdrawWindow(display, window, screen);
				// wait until unmapped
//				XIfEvent(display, &event, &is_unmap_notify, (XPointer)&window);
				XSync(display, False);
//				XFlush(display);
			}
		}
	}
}

void LedBox::CreateDialogBox()
{
	if (!enable) return;

	// Get X11 Display from SDL
	SDL_SysWMinfo sdl_info;
	SDL_VERSION(&sdl_info.version);
	SDL_GetWMInfo(&sdl_info);

	display = sdl_info.info.x11.display;
	if (!display) return;
	parent = sdl_info.info.x11.window;

	screen = DefaultScreen(display);

	unsigned long black;

	black = BlackPixel(display, screen);

	// Set size hint
	XSizeHints hints;
	hints.x = 0; hints.y = 0;
	hints.width = surface->w; hints.height = surface->h;

	// No Resizeable
	hints.base_width = hints.min_width = hints.max_width = hints.width + 2;
	hints.base_height = hints.min_height = hints.max_height = hints.height + 2;
	hints.width_inc = hints.height_inc = 0;

	hints.flags = PSize | PBaseSize | PMinSize | PMaxSize | PResizeInc; // | PPosition;

	// Create Window
	window = XCreateSimpleWindow(display
		, DefaultRootWindow(display)
//		, info.info.x11.window
		, hints.x, hints.y, hints.width, hints.height, 1, black, black);
	if (!window) return;

	// Set size hints
	XSetWMNormalHints(display, window, &hints);

	// Create Graphic Context
	gc = XCreateGC(display, window, 0, 0);
	XSetBackground(display, gc, black);
	XSetForeground(display, gc, black);

	// Attach pixel buffer to XImage structure.
	image.width = surface->w;
	image.height = surface->h;
	image.xoffset = 0;
	image.format = ZPixmap;
	image.data = (char *)surface->pixels;
	image.byte_order = (SDL_BYTEORDER == SDL_BIG_ENDIAN) ? MSBFirst : LSBFirst;
	image.bitmap_unit = surface->format->BitsPerPixel;
	image.bitmap_bit_order = (SDL_BYTEORDER == SDL_BIG_ENDIAN) ? MSBFirst : LSBFirst;
	image.bitmap_pad = surface->format->BitsPerPixel;
	image.depth = DefaultDepth(display, screen);
	image.bytes_per_line = 0;
	image.bits_per_pixel = surface->format->BitsPerPixel;
	image.red_mask = surface->format->Rmask;
	image.green_mask = surface->format->Gmask;
	image.blue_mask = surface->format->Bmask;
	XInitImage(&image);

	// Set Acceptable Event
	event_masks = Button1MotionMask | ButtonPressMask | ButtonReleaseMask | ExposureMask | StructureNotifyMask;
	XSelectInput(display, window, event_masks);

	// Mapping Window on display
	XMapWindow(display, window);

	// Wait Raised window
//	XFlush(display);

	// See "Xm/MwmUtils.h" about MOTIF definitions.
	#define MWM_HINTS_FUNCTIONS (1L << 0)
	#define MWM_HINTS_DECORATIONS (1L << 1)
	#define MWM_FUNC_CLOSE (1L << 5)
	#define MWM_DECOR_BORDER (1L << 1)

	// Send non border window styles to window manager using Motif property.
	Atom _MOTIF_WM_HINTS = XInternAtom(display, "_MOTIF_WM_HINTS", True);
	if (_MOTIF_WM_HINTS != None) {
		struct {
			unsigned long flags;
			unsigned long functions;
			unsigned long decorations;
			long input_mode;
			unsigned long status;
		} MWMHints = {
			MWM_HINTS_DECORATIONS,
			 0,
			 0,	// no border
			 0, 0
		};

		XChangeProperty(display, window,
			_MOTIF_WM_HINTS, _MOTIF_WM_HINTS, 32, PropModeReplace,
			(unsigned char *)&MWMHints, sizeof(MWMHints) / 4
		);
	}

	// accept event
	event_working = true;
}

void LedBox::destroy_dialog()
{
	event_working = false;

	if (gc) {
		XFreeGC(display, gc);
		gc = 0;
	}
	if (window) {
		XDestroyWindow(display, window);
		window = 0;
	}
}

/// process pending event
void LedBox::ProcessEvent()
{
	XEvent event;

	while (event_working
	&& XCheckWindowEvent(display, window, event_masks, &event)) {
		switch(event.type) {
		case Expose:
			if (event.xexpose.count == 0) {
				update_dialog();
			}
			break;
#ifdef NO_TITLEBAR
		case ButtonPress:
			if (event.xbutton.button == 1) {
				pStart.x = event.xbutton.x;
				pStart.y = event.xbutton.y;
			}
			break;
		case ButtonRelease:
			break;
		case MotionNotify:
			mouse_move(event);
			break;
#else
		case MotionNotify:
			set_dist();
			break;
#endif
		}
	}
}

Bool LedBox::is_map_notify(Display *d, XEvent *e, XPointer w)
{
	return e->type == MapNotify && e->xmap.window == *((Window *)w);
}

Bool LedBox::is_unmap_notify(Display *d, XEvent *e, XPointer w)
{
	return e->type == UnmapNotify && e->xunmap.window == *((Window *)w);
}

void LedBox::Move()
{
	XWindowAttributes pattr;

	if (window) {
		XGetWindowAttributes(display, parent, &pattr);

		int x = pattr.x + dist.x;
		int y = pattr.y + dist.y;

		XMoveWindow(display, window, x, y);
	}
}

void LedBox::move_in_place(int place)
{
	XWindowAttributes attr, pattr;

	if (window) {
		XGetWindowAttributes(display, window, &attr);
		XGetWindowAttributes(display, parent, &pattr);

		int w = attr.width;
		int h = attr.height;

		if (win_pt.place != place) {
			if (place & 1) {
				dist.x = pattr.width - w;
			} else {
				dist.x = 0;
			}
			if (place & 2) {
				dist.y = pattr.height - h;
			} else {
				dist.y = 0;
			}
		}

		int x = pattr.x + dist.x;
		int y = pattr.y + dist.y;

		XMoveWindow(display, window, x, y);
	}
}

#ifdef NO_TITLEBAR
void LedBox::mouse_move(const XEvent &event)
{
	XWindowAttributes attr, pattr;

	XGetWindowAttributes(display, window, &attr);
	XGetWindowAttributes(display, parent, &pattr);

	int x = event.xmotion.x - pStart.x + attr.x;
	int y = event.xmotion.y - pStart.y + attr.y;
	dist.x = attr.x - pattr.x;
	dist.y = attr.y - pattr.y;

	XMoveWindow(display, window, x, y);
}
#endif

#ifndef NO_TITLEBAR
void LedBox::set_dist()
{
	XWindowAttributes attr, pattr;

	XGetWindowAttributes(display, window, &attr);
	XGetWindowAttributes(display, parent, &pattr);

	dist.x = attr.x - pattr.x;
	dist.y = attr.y - pattr.y;
}
#endif

void LedBox::need_update_dialog()
{
	if (window) {
		XClearWindow(display, window); // expose event
	}
}

void LedBox::update_dialog()
{
	if (image.width <= 0 || image.height <= 0) return;

	XPutImage(display, window, gc, &image,
		0, 0, 1, 1,
		image.width, image.height
	);
}

#endif /* SDL_VIDEO_DRIVER_X11 && USE_X11_LEDBOX */
#endif /* !(USE_SDL2 && USE_SDL2_LEDBOX) */
