/** @file rec_video.h

	Skelton for retropc emulator

	@author Sasaji
	@date   2015.05.18 -

	@brief [ record video ]
*/

#ifndef _RECORD_VIDEO_H_
#define _RECORD_VIDEO_H_

#include "../common.h"
#include <SDL.h>
#ifdef USE_REC_VIDEO_VFW
class VFW_REC_VIDEO;
#endif
#ifdef USE_REC_VIDEO_QTKIT
class QT_REC_VIDEO;
#endif
#ifdef USE_REC_VIDEO_FFMPEG
class FFM_REC_VIDEO;
#endif
#ifdef USE_REC_VIDEO_AVKIT
class AVK_REC_VIDEO;
#endif
#ifdef USE_REC_VIDEO_WIN
class WIN_REC_VIDEO;
#endif
#ifdef USE_REC_VIDEO_COCOA
class COCOA_REC_VIDEO;
#endif
#ifdef USE_REC_VIDEO_LIBPNG
class PNG_REC_VIDEO;
#endif

enum en_record_video_type {
	RECORD_VIDEO_TYPE_VFW = 1,
	RECORD_VIDEO_TYPE_QTKIT,
	RECORD_VIDEO_TYPE_FFMPEG,
	RECORD_VIDEO_TYPE_AVKIT,
	CAPTURE_SCREEN_TYPE = 11,
	SAVE_IMAGE_TYPE,
	RECORD_VIDEO_TYPE_UNKNOWN
};

class EMU;

class REC_VIDEO
{
private:
	EMU *emu;
	bool now_recording;
	int rec_type;
	int rec_fps;
	int rec_count;
	int rec_denomi;
	_TCHAR rec_path[_MAX_PATH];
#ifdef USE_REC_VIDEO_VFW
	VFW_REC_VIDEO *vfwvideo;
#endif
#ifdef USE_REC_VIDEO_QTKIT
	QT_REC_VIDEO *qtvideo;
#endif
#ifdef USE_REC_VIDEO_FFMPEG
	FFM_REC_VIDEO *ffmvideo;
#endif
#ifdef USE_REC_VIDEO_AVKIT
	AVK_REC_VIDEO *avkvideo;
#endif
#ifdef USE_REC_VIDEO_WIN
	WIN_REC_VIDEO *winvideo;
#endif
#ifdef USE_REC_VIDEO_COCOA
	COCOA_REC_VIDEO *cocvideo;
#endif
#ifdef USE_REC_VIDEO_LIBPNG
	PNG_REC_VIDEO *pngvideo;
#endif
	SDL_Surface *rec_surface;

	bool SaveBMP(SDL_Surface *surface, const char *file_name);

public:
	REC_VIDEO(EMU *new_emu);
	~REC_VIDEO();

	/// true:OK false:ERROR
	bool Start(int type, int fps, SDL_Rect *srcrect, SDL_Surface *srcsurface, bool show_dialog);
	void Stop();
	bool Restart();
	bool Record(SDL_Rect *srcrect, SDL_Surface *srcsurface, SDL_Rect *dstrect);
	bool IsRecordFrame();
	bool NowRecording() {
		return now_recording;
	}
	bool *GetNowRecordingPtr() {
		return &now_recording;
	}
	bool Capture(int type, SDL_Rect *srcrect, SDL_Surface *srcsurface, SDL_Rect *dstrect);
	void CreateFileName(_TCHAR *file_path);

	bool IsEnabled(int type);
	const _TCHAR **GetCodecList(int type);
	const _TCHAR **GetQualityList(int type);

};

#endif /* _RECORD_VIDEO_H_ */
