/** @file cmt.h

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	Skelton for retropc emulator

	@author Sasaji
	@date   2011.06.13 -

	@brief [ cmt ]
*/

#ifndef _CMT_H_
#define _CMT_H_

#include "../vm.h"
#include "../../emu.h"
#include "../device.h"
#include "acia.h"
#include "parsewav.h"

#define CMTDATA_BUFFER_SIZE	1024
#define WAVE_PARSE_SIZE		170
#define SIG_CMT_REMOTE		1
#define SIG_CMT_BAUD_SEL	2

#define TAPEWAV_BUFFER_SIZE	2048

class FILEIO;
class PARSEWAV;

/**
	@brief Cassette Magnetic Tape
*/
class CMT : public DEVICE
{
private:
	DEVICE* d_ctrl;

	uint8  data_buffer[CMTDATA_BUFFER_SIZE];
	char   c_onedata[CMTDATA_BUFFER_SIZE];
	int    data_read_pos;
	int    data_read_size;
	int    data_write_pos;
	int    data_write_size;
	int	   data_total_pos;
	bool   buffer_overflow;
	int    file_type;	// 0:wav 1:l3c 2:l3c 3:l3 4:t9x
	bool   is_bytedata;	// .l3 = true
	bool   need_header;
	int    data_len;

	/// This format is undocumented.
	typedef struct t9x_header_st {
		char ident[32];
		int  data1;
		int  data2;
		char reserved[24];
	} t9x_header_t;
	t9x_header_t t9x_header;
	uint8  t9x_data;
	uint8  t9x_dlen;

	FILEIO   *fio;
	PARSEWAV *wav;
	bool play, rec;
	bool remote;		// remote control
	bool remote_prev;
	uint8 baud_sel;		// baud select 0:600 1:1200
	int  baud_rate;
	bool send_ok;		// send to acia
	bool recv_ok;		// receive from acia

	int sample_rate_base;
	int sample_rate;
	int max_vol;
	int relay_volume;
	int tape_volume;
	int relay_wav_enable;
	bool relay_wav_loaded_at_first;
	_TCHAR relay_file[2][16];
	uint8 *relay_data[2];
	int    relay_size[2];
	int    relay_play_pos;
	int    relay_play;

	int16 *tape_wave_buf;
	int   tape_wave_buf_wpos;
	int   tape_wave_buf_rpos;
	int   tape_wave_nums;
	int   tape_wave_onoff;

	int   devide;
	int   surplus;
	int   freq_prev;
	int   a_prev;

	int register_id[2];

	//for resume
#pragma pack(1)
	struct vm_state_st {
		int register_id;
		uint8 flags;
		uint8 baud_sel; // add version 3

		char  reserved[10];
	};
#pragma pack()
	bool  set_load_param();
	bool  set_save_param();
	void  load_image();
	void  save_image();
	void	set_data(uint32);
	uint32	get_data();
	uint32	get_data_fast();
	int  check_extension(const _TCHAR *);
	int  check_file_format(FILEIO *, int);
	int  set_file_header(FILEIO *, int);

	void register_my_event(int);
	void cancel_my_event(int);

	void load_relay_wav();

	void set_baud(int sel);

	void create_tape_wave();
	void set_tape_wave(uint32 data);
	void set_tape_one_wave(int cnt, int samp);
	void set_tape_amp(char *c_data);
	void set_tape_one_amp(int data);

public:
	CMT(VM* parent_vm, EMU* parent_emu, const char* identifier) : DEVICE(parent_vm, parent_emu, identifier) {
		set_class_name("CMT");
		d_ctrl = NULL;
	}
	~CMT() {}

	// common functions
	void initialize();
	void reset();
	void release();

	void write_io8(uint32 addr, uint32 data);
	uint32 read_io8(uint32 addr);
	void write_signal(int id, uint32 data, uint32 mask);

	// unique functions
	void set_context_ctrl(DEVICE* device) {
		d_ctrl = device;
	}
	bool play_datarec(const _TCHAR* filename);
	bool rec_datarec(const _TCHAR* filename);
	void close_datarec();
	void rewind_datarec();
	void fast_forward_datarec();
	void realmode_datarec();
	bool datarec_opened(bool play_mode);

	void event_callback(int , int);
	void mix(int32* buffer, int cnt);
	void set_volume(int volr, int volt);
	void init(int rate, int volume);

	uint32 get_cmt_mode();

	void update_config();

	void save_state(FILEIO *fp);
	void load_state(FILEIO *fp);

#ifdef USE_DEBUGGER
	bool debug_write_reg(uint32 reg_num, uint32 data);
	void debug_regs_info(_TCHAR *buffer, size_t buffer_len);
#endif
};

#endif
